package org.rogmann.jsmud.source;

import org.rogmann.jsmud.shadow.asm.Opcodes;
import org.rogmann.jsmud.shadow.asm.tree.ClassNode;
import org.rogmann.jsmud.shadow.asm.tree.MethodInsnNode;

/**
 * Method-instruction which executes a void method as a statement.
 */
public class StatementInvoke extends StatementInstr<MethodInsnNode> {

	/** current class */
	protected final ClassNode classNode;

	/** expression of object-instance */
	private final ExpressionBase<?> exprObj;
	/** arguments of constructor */
	private final ExpressionBase<?>[] exprArgs;

	/**
	 * Constructor
	 * @param insn variable-instruction, e.g. ASTORE_1
	 * @param classNode node of current class
	 * @param exprObj expression of object-instance (<code>null</code> in case of INVOKESTATIC)
	 * @param exprArgs arguments of method
	 */
	public StatementInvoke(MethodInsnNode insn, ClassNode classNode, final ExpressionBase<?> exprObj,
			final ExpressionBase<?>... exprArgs) {
		super(insn);
		this.classNode = classNode;
		this.exprObj = exprObj;
		this.exprArgs = exprArgs;
	}

	/** {@inheritDoc} */
	@Override
	public void render(StringBuilder sb) {
		if (insn.getOpcode() == Opcodes.INVOKESTATIC) {
			final String className = insn.owner.replace('/', '.');
			sb.append(SourceFileWriter.simplifyClassName(className));
		}
		else {
			exprObj.render(sb);
		}
		sb.append('.');
		sb.append(insn.name);
		sb.append('(');
		boolean isFirst = true;
		for (final ExpressionBase<?> arg : exprArgs) {
			if (isFirst) {
				isFirst = false;
			}
			else {
				sb.append(", ");
			}
			arg.render(sb);
		}
		sb.append(')');
		sb.append(';');
	}

}
